import asyncio
from typing import Any, Coroutine, Optional, Tuple, Type

from grpc import aio
from grpc_reflection.v1alpha import reflection

from jetpack.config import _symbols
from jetpack.core._jetpack_client import default_jetpack_client
from jetpack.proto.runtime.v1alpha1 import jetworker_pb2, jetworker_pb2_grpc


async def start_grpc_server(port: str) -> None:

    print(f"starting the grpc server on port {port}")
    server = aio.server()  # grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    jetworker_pb2_grpc.add_JetworkerServicer_to_server(JetworkerServicer(), server)

    # ref. https://github.com/grpc/grpc/blob/master/doc/python/server_reflection.md
    service_names = (
        jetworker_pb2.DESCRIPTOR.services_by_name["Jetworker"].full_name,
        reflection.SERVICE_NAME,
    )
    reflection.enable_server_reflection(service_names, server)

    server.add_insecure_port(f"[::]:{port}")
    await server.start()
    print("SUCCESS: started the grpc server")
    await server.wait_for_termination()


class JetworkerServicer(jetworker_pb2_grpc.JetworkerServicer):

    # Need to figure out grpc request types
    #
    # getting nox-py2 error for typechecking:
    #
    # Return type "Coroutine[Any, Any, StartJetroutineResponse]" of "StartJetroutine"
    # incompatible with return type "StartJetroutineResponse" in supertype "JetworkerServicer"
    async def StartJetroutine(  # type: ignore
        self, request: jetworker_pb2.StartJetroutineRequest, context: Any
    ) -> jetworker_pb2.StartJetroutineResponse:
        func = _symbols.get_symbol_table().get_registered_symbols()[
            request.qualified_symbol
        ]

        args = b""
        if request.encoded_args:
            args = request.encoded_args.encode()

        await default_jetpack_client.jetroutine(func)._exec(request.exec_id, args)
        return jetworker_pb2.StartJetroutineResponse()



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

APPLICATION: ErrorCode
DAYS: Unit
DESCRIPTOR: _descriptor.FileDescriptor
FRIDAY: DayOfWeek
HOURS: Unit
MINUTES: Unit
MONDAY: DayOfWeek
SATURDAY: DayOfWeek
SECONDS: Unit
SUNDAY: DayOfWeek
SYSTEM: ErrorCode
THURSDAY: DayOfWeek
TUESDAY: DayOfWeek
UNKNOWN: ErrorCode
UNKNOWN_DAY: DayOfWeek
UNKNOWN_UNIT: Unit
WEDNESDAY: DayOfWeek
WEEKS: Unit

class CancelTaskRequest(_message.Message):
    __slots__ = ["task_id"]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    task_id: str
    def __init__(self, task_id: _Optional[str] = ...) -> None: ...

class CancelTaskResponse(_message.Message):
    __slots__ = ["success"]
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    success: bool
    def __init__(self, success: bool = ...) -> None: ...

class CreateTaskRequest(_message.Message):
    __slots__ = ["sdk_version", "task"]
    SDK_VERSION_FIELD_NUMBER: _ClassVar[int]
    TASK_FIELD_NUMBER: _ClassVar[int]
    sdk_version: str
    task: Task
    def __init__(self, task: _Optional[_Union[Task, _Mapping]] = ..., sdk_version: _Optional[str] = ...) -> None: ...

class CreateTaskResponse(_message.Message):
    __slots__ = ["task_id"]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    task_id: str
    def __init__(self, task_id: _Optional[str] = ...) -> None: ...

class CronJob(_message.Message):
    __slots__ = ["interval", "qualified_symbol", "schedule_rule", "target_day_of_week", "target_time", "unit"]
    INTERVAL_FIELD_NUMBER: _ClassVar[int]
    QUALIFIED_SYMBOL_FIELD_NUMBER: _ClassVar[int]
    SCHEDULE_RULE_FIELD_NUMBER: _ClassVar[int]
    TARGET_DAY_OF_WEEK_FIELD_NUMBER: _ClassVar[int]
    TARGET_TIME_FIELD_NUMBER: _ClassVar[int]
    UNIT_FIELD_NUMBER: _ClassVar[int]
    interval: int
    qualified_symbol: str
    schedule_rule: str
    target_day_of_week: DayOfWeek
    target_time: str
    unit: Unit
    def __init__(self, qualified_symbol: _Optional[str] = ..., target_time: _Optional[str] = ..., target_day_of_week: _Optional[_Union[DayOfWeek, str]] = ..., unit: _Optional[_Union[Unit, str]] = ..., interval: _Optional[int] = ..., schedule_rule: _Optional[str] = ...) -> None: ...

class DeregisterAppRequest(_message.Message):
    __slots__ = ["hostname", "namespace"]
    HOSTNAME_FIELD_NUMBER: _ClassVar[int]
    NAMESPACE_FIELD_NUMBER: _ClassVar[int]
    hostname: str
    namespace: str
    def __init__(self, namespace: _Optional[str] = ..., hostname: _Optional[str] = ...) -> None: ...

class DeregisterAppResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class Error(_message.Message):
    __slots__ = ["code", "encoded_error", "message"]
    CODE_FIELD_NUMBER: _ClassVar[int]
    ENCODED_ERROR_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    code: ErrorCode
    encoded_error: bytes
    message: str
    def __init__(self, code: _Optional[_Union[ErrorCode, str]] = ..., message: _Optional[str] = ..., encoded_error: _Optional[bytes] = ...) -> None: ...

class GetSymbolFromEndpointRequest(_message.Message):
    __slots__ = ["endpoint"]
    ENDPOINT_FIELD_NUMBER: _ClassVar[int]
    endpoint: str
    def __init__(self, endpoint: _Optional[str] = ...) -> None: ...

class GetSymbolFromEndpointResponse(_message.Message):
    __slots__ = ["symbol"]
    SYMBOL_FIELD_NUMBER: _ClassVar[int]
    symbol: str
    def __init__(self, symbol: _Optional[str] = ...) -> None: ...

class GetTaskRequest(_message.Message):
    __slots__ = ["task_id"]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    task_id: str
    def __init__(self, task_id: _Optional[str] = ...) -> None: ...

class GetTaskResponse(_message.Message):
    __slots__ = ["encoded_args", "id"]
    ENCODED_ARGS_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    encoded_args: bytes
    id: str
    def __init__(self, id: _Optional[str] = ..., encoded_args: _Optional[bytes] = ...) -> None: ...

class PostResultRequest(_message.Message):
    __slots__ = ["exec_id", "result"]
    EXEC_ID_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    exec_id: str
    result: Result
    def __init__(self, exec_id: _Optional[str] = ..., result: _Optional[_Union[Result, _Mapping]] = ...) -> None: ...

class PostResultResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class RegisterAppRequest(_message.Message):
    __slots__ = ["cron_jobs", "endpoints", "hostname", "namespace", "qualified_symbols"]
    class EndpointsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    CRON_JOBS_FIELD_NUMBER: _ClassVar[int]
    ENDPOINTS_FIELD_NUMBER: _ClassVar[int]
    HOSTNAME_FIELD_NUMBER: _ClassVar[int]
    NAMESPACE_FIELD_NUMBER: _ClassVar[int]
    QUALIFIED_SYMBOLS_FIELD_NUMBER: _ClassVar[int]
    cron_jobs: _containers.RepeatedCompositeFieldContainer[CronJob]
    endpoints: _containers.ScalarMap[str, str]
    hostname: str
    namespace: str
    qualified_symbols: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, namespace: _Optional[str] = ..., hostname: _Optional[str] = ..., cron_jobs: _Optional[_Iterable[_Union[CronJob, _Mapping]]] = ..., qualified_symbols: _Optional[_Iterable[str]] = ..., endpoints: _Optional[_Mapping[str, str]] = ...) -> None: ...

class RegisterAppResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class RemoteCallRequest(_message.Message):
    __slots__ = ["json_args", "qualified_symbol"]
    JSON_ARGS_FIELD_NUMBER: _ClassVar[int]
    QUALIFIED_SYMBOL_FIELD_NUMBER: _ClassVar[int]
    json_args: str
    qualified_symbol: str
    def __init__(self, qualified_symbol: _Optional[str] = ..., json_args: _Optional[str] = ...) -> None: ...

class RemoteCallResponse(_message.Message):
    __slots__ = ["json_results"]
    JSON_RESULTS_FIELD_NUMBER: _ClassVar[int]
    json_results: str
    def __init__(self, json_results: _Optional[str] = ...) -> None: ...

class Result(_message.Message):
    __slots__ = ["error", "value"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    error: Error
    value: Value
    def __init__(self, value: _Optional[_Union[Value, _Mapping]] = ..., error: _Optional[_Union[Error, _Mapping]] = ...) -> None: ...

class Task(_message.Message):
    __slots__ = ["app_name", "chart_revision", "encoded_args", "hostname", "namespace", "parent_task_id", "qualified_symbol", "target_time", "task_id", "with_checkpointing"]
    APP_NAME_FIELD_NUMBER: _ClassVar[int]
    CHART_REVISION_FIELD_NUMBER: _ClassVar[int]
    ENCODED_ARGS_FIELD_NUMBER: _ClassVar[int]
    HOSTNAME_FIELD_NUMBER: _ClassVar[int]
    NAMESPACE_FIELD_NUMBER: _ClassVar[int]
    PARENT_TASK_ID_FIELD_NUMBER: _ClassVar[int]
    QUALIFIED_SYMBOL_FIELD_NUMBER: _ClassVar[int]
    TARGET_TIME_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    WITH_CHECKPOINTING_FIELD_NUMBER: _ClassVar[int]
    app_name: str
    chart_revision: str
    encoded_args: bytes
    hostname: str
    namespace: str
    parent_task_id: str
    qualified_symbol: str
    target_time: _timestamp_pb2.Timestamp
    task_id: str
    with_checkpointing: bool
    def __init__(self, namespace: _Optional[str] = ..., hostname: _Optional[str] = ..., app_name: _Optional[str] = ..., chart_revision: _Optional[str] = ..., qualified_symbol: _Optional[str] = ..., encoded_args: _Optional[bytes] = ..., parent_task_id: _Optional[str] = ..., target_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., with_checkpointing: bool = ..., task_id: _Optional[str] = ...) -> None: ...

class Value(_message.Message):
    __slots__ = ["encoded_value"]
    ENCODED_VALUE_FIELD_NUMBER: _ClassVar[int]
    encoded_value: bytes
    def __init__(self, encoded_value: _Optional[bytes] = ...) -> None: ...

class WaitForResultRequest(_message.Message):
    __slots__ = ["task_id"]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    task_id: str
    def __init__(self, task_id: _Optional[str] = ...) -> None: ...

class WaitForResultResponse(_message.Message):
    __slots__ = ["result"]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    result: Result
    def __init__(self, result: _Optional[_Union[Result, _Mapping]] = ...) -> None: ...

class ErrorCode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class Unit(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []

class DayOfWeek(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []



from typing import Awaitable, Callable, TypeVar, Union

from jetpack.core._jetroutine import Jetroutine
from jetpack.runtime._client import RuntimeClient, default_runtime_client

T = TypeVar("T")


class JetpackClient:
    def __init__(
        self,
        runtimeClient: RuntimeClient,
    ) -> None:
        self.runtimeClient = runtimeClient

    def jetroutine(
        self,
        func: Callable[..., Union[T, Awaitable[T]]],
        with_checkpointing: bool = False,
    ) -> Jetroutine[T]:
        jetroutine: Jetroutine[T] = Jetroutine(
            self.runtimeClient, func, with_checkpointing
        )
        return jetroutine


default_jetpack_client = JetpackClient(default_runtime_client)



from jetpack.proto.runtime.v1alpha1 import remote_pb2 as _remote_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class UserApp(_message.Message):
    __slots__ = ["latest_manifest", "revision_to_manifest"]
    class RevisionToManifestEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: int
        value: str
        def __init__(self, key: _Optional[int] = ..., value: _Optional[str] = ...) -> None: ...
    LATEST_MANIFEST_FIELD_NUMBER: _ClassVar[int]
    REVISION_TO_MANIFEST_FIELD_NUMBER: _ClassVar[int]
    latest_manifest: str
    revision_to_manifest: _containers.ScalarMap[int, str]
    def __init__(self, revision_to_manifest: _Optional[_Mapping[int, str]] = ..., latest_manifest: _Optional[str] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: jetpack/proto/runtime/v1alpha1/task.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from jetpack.proto.runtime.v1alpha1 import remote_pb2 as jetpack_dot_proto_dot_runtime_dot_v1alpha1_dot_remote__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)jetpack/proto/runtime/v1alpha1/task.proto\x12\x04task\x1a+jetpack/proto/runtime/v1alpha1/remote.proto\"\x9c\x05\n\rPersistedTask\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12)\n\x10qualified_symbol\x18\x02 \x01(\tR\x0fqualifiedSymbol\x12!\n\x0c\x65ncoded_args\x18\x03 \x01(\x0cR\x0b\x65ncodedArgs\x12\x1a\n\x08manifest\x18\x04 \x01(\x0cR\x08manifest\x12\x32\n\x06status\x18\x05 \x01(\x0e\x32\x1a.task.PersistedTask.StatusR\x06status\x12&\n\x06result\x18\x06 \x01(\x0b\x32\x0e.remote.ResultR\x06result\x12\x19\n\x08\x61pp_name\x18\x07 \x01(\tR\x07\x61ppName\x12$\n\x0eparent_task_id\x18\x08 \x01(\tR\x0cparentTaskId\x12-\n\x12with_checkpointing\x18\t \x01(\x08R\x11withCheckpointing\x12+\n\x11\x63urrent_iteration\x18\n \x01(\x05R\x10\x63urrentIteration\x12H\n\x0e\x65xecution_mode\x18\x0b \x01(\x0e\x32!.task.PersistedTask.ExecutionModeR\rexecutionMode\"\x8e\x01\n\x06Status\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07\x43REATED\x10\x01\x12\x0b\n\x07WAITING\x10\x02\x12\t\n\x05READY\x10\x03\x12\x0b\n\x07RUNNING\x10\x04\x12\r\n\tSUCCEEDED\x10\x05\x12\x0e\n\nCANCELLING\x10\x06\x12\r\n\tCANCELLED\x10\x07\x12\x0b\n\x07\x46\x41ILING\x10\x08\x12\n\n\x06\x46\x41ILED\x10\t\"=\n\rExecutionMode\x12\x11\n\rEXECUTION_JOB\x10\x00\x12\x19\n\x15\x45XECUTION_WORKER_PUSH\x10\x01\"\xaf\x01\n\x11\x43heckpointTracker\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12J\n\x0b\x63heckpoints\x18\x02 \x03(\x0b\x32(.task.CheckpointTracker.CheckpointsEntryR\x0b\x63heckpoints\x1a>\n\x10\x43heckpointsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42y\n\x08\x63om.taskB\tTaskProtoP\x01Z2go.jetpack.io/proto/jetpack/proto/runtime/v1alpha1\xa2\x02\x03TXX\xaa\x02\x04Task\xca\x02\x04Task\xe2\x02\x10Task\\GPBMetadata\xea\x02\x04Taskb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'jetpack.proto.runtime.v1alpha1.task_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\010com.taskB\tTaskProtoP\001Z2go.jetpack.io/proto/jetpack/proto/runtime/v1alpha1\242\002\003TXX\252\002\004Task\312\002\004Task\342\002\020Task\\GPBMetadata\352\002\004Task'
  _CHECKPOINTTRACKER_CHECKPOINTSENTRY._options = None
  _CHECKPOINTTRACKER_CHECKPOINTSENTRY._serialized_options = b'8\001'
  _PERSISTEDTASK._serialized_start=97
  _PERSISTEDTASK._serialized_end=765
  _PERSISTEDTASK_STATUS._serialized_start=560
  _PERSISTEDTASK_STATUS._serialized_end=702
  _PERSISTEDTASK_EXECUTIONMODE._serialized_start=704
  _PERSISTEDTASK_EXECUTIONMODE._serialized_end=765
  _CHECKPOINTTRACKER._serialized_start=768
  _CHECKPOINTTRACKER._serialized_end=943
  _CHECKPOINTTRACKER_CHECKPOINTSENTRY._serialized_start=881
  _CHECKPOINTTRACKER_CHECKPOINTSENTRY._serialized_end=943
# @@protoc_insertion_point(module_scope)



from jetpack.proto.runtime.v1alpha1 import remote_pb2 as _remote_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class CheckpointTracker(_message.Message):
    __slots__ = ["checkpoints", "id"]
    class CheckpointsEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    CHECKPOINTS_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    checkpoints: _containers.ScalarMap[str, str]
    id: str
    def __init__(self, id: _Optional[str] = ..., checkpoints: _Optional[_Mapping[str, str]] = ...) -> None: ...

class PersistedTask(_message.Message):
    __slots__ = ["app_name", "current_iteration", "encoded_args", "execution_mode", "id", "manifest", "parent_task_id", "qualified_symbol", "result", "status", "with_checkpointing"]
    class ExecutionMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    class Status(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
    APP_NAME_FIELD_NUMBER: _ClassVar[int]
    CANCELLED: PersistedTask.Status
    CANCELLING: PersistedTask.Status
    CREATED: PersistedTask.Status
    CURRENT_ITERATION_FIELD_NUMBER: _ClassVar[int]
    ENCODED_ARGS_FIELD_NUMBER: _ClassVar[int]
    EXECUTION_JOB: PersistedTask.ExecutionMode
    EXECUTION_MODE_FIELD_NUMBER: _ClassVar[int]
    EXECUTION_WORKER_PUSH: PersistedTask.ExecutionMode
    FAILED: PersistedTask.Status
    FAILING: PersistedTask.Status
    ID_FIELD_NUMBER: _ClassVar[int]
    MANIFEST_FIELD_NUMBER: _ClassVar[int]
    PARENT_TASK_ID_FIELD_NUMBER: _ClassVar[int]
    QUALIFIED_SYMBOL_FIELD_NUMBER: _ClassVar[int]
    READY: PersistedTask.Status
    RESULT_FIELD_NUMBER: _ClassVar[int]
    RUNNING: PersistedTask.Status
    STATUS_FIELD_NUMBER: _ClassVar[int]
    SUCCEEDED: PersistedTask.Status
    UNKNOWN: PersistedTask.Status
    WAITING: PersistedTask.Status
    WITH_CHECKPOINTING_FIELD_NUMBER: _ClassVar[int]
    app_name: str
    current_iteration: int
    encoded_args: bytes
    execution_mode: PersistedTask.ExecutionMode
    id: str
    manifest: bytes
    parent_task_id: str
    qualified_symbol: str
    result: _remote_pb2.Result
    status: PersistedTask.Status
    with_checkpointing: bool
    def __init__(self, id: _Optional[str] = ..., qualified_symbol: _Optional[str] = ..., encoded_args: _Optional[bytes] = ..., manifest: _Optional[bytes] = ..., status: _Optional[_Union[PersistedTask.Status, str]] = ..., result: _Optional[_Union[_remote_pb2.Result, _Mapping]] = ..., app_name: _Optional[str] = ..., parent_task_id: _Optional[str] = ..., with_checkpointing: bool = ..., current_iteration: _Optional[int] = ..., execution_mode: _Optional[_Union[PersistedTask.ExecutionMode, str]] = ...) -> None: ...



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class StartJetroutineRequest(_message.Message):
    __slots__ = ["encoded_args", "exec_id", "qualified_symbol"]
    ENCODED_ARGS_FIELD_NUMBER: _ClassVar[int]
    EXEC_ID_FIELD_NUMBER: _ClassVar[int]
    QUALIFIED_SYMBOL_FIELD_NUMBER: _ClassVar[int]
    encoded_args: str
    exec_id: str
    qualified_symbol: str
    def __init__(self, exec_id: _Optional[str] = ..., qualified_symbol: _Optional[str] = ..., encoded_args: _Optional[str] = ...) -> None: ...

class StartJetroutineResponse(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...



import asyncio
from concurrent import futures
import os
import sys
import time
from types import TracebackType
from typing import Any, Optional, Tuple, Type

import click

from jetpack import cron
from jetpack.cmd import _util
from jetpack.cmd._cron import cron_group
from jetpack.cmd._params import ENTRYPOINT_PARAMS
from jetpack.config import _symbols
from jetpack.core._jetpack_client import default_jetpack_client
from jetpack.runtime._client import default_runtime_client
from jetpack.server._server import start_grpc_server

_original_excepthook = sys.excepthook


def handle_sys_excepthook(
    exc_type: Type[BaseException],
    exc_value: BaseException,
    exc_traceback: Optional[TracebackType],
) -> Any:
    """
    This function handles the empty sys.excepthook error.
    One possible explanation is that module members are set to `None` during module cleanup.
    At the time when `sys.excepthook` is called, all module members are already set to `None`.
    And since python doesn't guarantee the order of modules being cleaned up,
    modules accessing a cleaned up module will result in an uncaught exception of empty.
    This function re-invokes the original sys.excepthook upon such error.
    """
    _original_excepthook(exc_type, exc_value, exc_traceback)


sys.excepthook = handle_sys_excepthook


_using_new_cli = False


def is_using_new_cli() -> bool:
    """
    for legacy uses, we keep old cli. This function disables that logic to ensure
    we don't run the cli command twice.
    """
    return _using_new_cli


@click.group()
def cli() -> None:
    global _using_new_cli
    _using_new_cli = True


@click.command(help="Executes jetpack task")
@click.option("--entrypoint", **ENTRYPOINT_PARAMS)
@click.option("--exec-id", required=True)
@click.option("--qualified-symbol", required=True)
@click.option("--encoded-args", default=None)
def exec_task(
    entrypoint: str,
    exec_id: str,
    qualified_symbol: str,
    encoded_args: str,  # jsonpickle encoded string
) -> None:
    _util.load_user_entrypoint(entrypoint)
    func = _symbols.get_symbol_table().get_registered_symbols()[qualified_symbol]

    async def _inner_exec_task() -> Tuple[Optional[Any], Optional[Exception]]:
        args = b""
        if encoded_args:
            args = encoded_args.encode()
        else:
            response = await default_runtime_client.get_task(exec_id)
            args = response.encoded_args

        return await default_jetpack_client.jetroutine(func)._exec(exec_id, args)

    asyncio.run(_inner_exec_task())


@click.command(help="Executes a cronjob or jetroutine")
@click.option("--entrypoint", **ENTRYPOINT_PARAMS)
@click.option("--qualified-symbol", required=True)
@click.option("--match-suffix", is_flag=True, default=False)
def exec_cronjob(entrypoint: str, qualified_symbol: str, match_suffix: bool) -> None:
    _util.load_user_entrypoint(entrypoint)
    symbols_table = _symbols.get_symbol_table()

    if match_suffix:
        func = symbols_table.get_registered_symbols().get(qualified_symbol)
        if func is None:
            # Try fuzzy matching.
            all_symbols = symbols_table.defined_symbols()
            matching_funcs = [x for x in all_symbols if x.endswith(qualified_symbol)]
            num_funcs = len(matching_funcs)

            if num_funcs == 1:
                func = symbols_table.get_registered_symbols()[matching_funcs[0]]
            else:
                if not num_funcs:
                    print(
                        f"Unable to find function {qualified_symbol} in your project."
                    )
                else:
                    print(
                        f"Found multiple functions with name {qualified_symbol} in your project."
                    )
                    print(
                        "Did you mean any of the following:\n\t"
                        + "\n\t".join(matching_funcs)
                    )
                return
    else:
        # Fail and throw error if key is not found in table.
        func = symbols_table.get_registered_symbols()[qualified_symbol]
    asyncio.run(default_jetpack_client.jetroutine(func)._exec(post_result=False))


@click.command(help="Registers jetpack functions with runtime")
@click.option("--entrypoint", **ENTRYPOINT_PARAMS)
def register(entrypoint: str) -> None:
    async def _register() -> None:
        _util.load_user_entrypoint(entrypoint)
        await default_runtime_client.register_app(cron.get_jobs())

    asyncio.run(_register())


@click.command(help="De-registers jetpack functions with runtime")
def deregister() -> None:
    async def _deregister() -> None:
        await default_runtime_client.deregister_app()

    asyncio.run(_deregister())


@click.command(help="List jetroutines")
@click.option("--entrypoint", **ENTRYPOINT_PARAMS)
def ls(entrypoint: str) -> None:
    _util.load_user_entrypoint(entrypoint)
    # print each symbol on its own line
    print("\n".join(_symbols.get_symbol_table().defined_symbols()))


@click.command(help="start jetworker server")
@click.option("--port", default="8081", type=str)
def jetworker_server(port: str) -> None:
    print("loading entrypoint")
    # we need to convert to string, because the ENTRYPOINT_PARAMS values are
    # typed to be objects
    entrypoint = os.environ.get(
        str(ENTRYPOINT_PARAMS["envvar"]), str(ENTRYPOINT_PARAMS["default"])
    )

    _util.load_user_entrypoint(entrypoint)
    asyncio.run(start_grpc_server(port))


cli.add_command(exec_task)
cli.add_command(exec_cronjob)
cli.add_command(register)
cli.add_command(deregister)
cli.add_command(ls)
cli.add_command(cron_group)
cli.add_command(jetworker_server)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: jetpack/proto/runtime/v1alpha1/jetworker.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.jetpack/proto/runtime/v1alpha1/jetworker.proto\x12\tjetworker\"\x7f\n\x16StartJetroutineRequest\x12\x17\n\x07\x65xec_id\x18\x01 \x01(\tR\x06\x65xecId\x12)\n\x10qualified_symbol\x18\x02 \x01(\tR\x0fqualifiedSymbol\x12!\n\x0c\x65ncoded_args\x18\x03 \x01(\tR\x0b\x65ncodedArgs\"\x19\n\x17StartJetroutineResponse2g\n\tJetworker\x12Z\n\x0fStartJetroutine\x12!.jetworker.StartJetroutineRequest\x1a\".jetworker.StartJetroutineResponse\"\x00\x42\x97\x01\n\rcom.jetworkerB\x0eJetworkerProtoP\x01Z2go.jetpack.io/proto/jetpack/proto/runtime/v1alpha1\xa2\x02\x03JXX\xaa\x02\tJetworker\xca\x02\tJetworker\xe2\x02\x15Jetworker\\GPBMetadata\xea\x02\tJetworkerb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'jetpack.proto.runtime.v1alpha1.jetworker_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\rcom.jetworkerB\016JetworkerProtoP\001Z2go.jetpack.io/proto/jetpack/proto/runtime/v1alpha1\242\002\003JXX\252\002\tJetworker\312\002\tJetworker\342\002\025Jetworker\\GPBMetadata\352\002\tJetworker'
  _STARTJETROUTINEREQUEST._serialized_start=61
  _STARTJETROUTINEREQUEST._serialized_end=188
  _STARTJETROUTINERESPONSE._serialized_start=190
  _STARTJETROUTINERESPONSE._serialized_end=215
  _JETWORKER._serialized_start=217
  _JETWORKER._serialized_end=320
# @@protoc_insertion_point(module_scope)



"""
Submodule for scheduling and running functions as Kubernetes CronJobs
"""
import os
from typing import Any, Callable, List, Optional

from schedule import Job as ScheduleJob  # Use this to whitelist what we allow
from schedule import every

from jetpack.config import _instrumentation, _symbols
from jetpack.proto.runtime.v1alpha1 import remote_pb2

cronjob_suffix = os.environ.get("JETPACK_CRONJOB_SUFFIX", "-missing-suffix")


def repeat(
    repeat_pattern: Optional[ScheduleJob] = None, schedule: Optional[str] = None
) -> Callable[..., Any]:
    """Decorate a function to run it as a Kubernetes CronJob.

    The decorated function will be registered with the runtime, which will create a
    CronJob object on your cluster, with the schedule set to match the repeat_pattern
    or schedule provided as an argument.

    Typical Examples:

        # Run my_function daily

        @cron.repeat(schedule="0 0 * * *")
        def my_function():

        # Run my_function every 2 hours

        @cron.repeat(cron.every(2).hours)
        def my_function():

    Args:
        repeat_pattern (schedule.Job): Defines the CronJob schedule using the `schedule.Job` API
        schedule (str): crontab format (e.g. 1 2 3 * *)

    Returns:
        Callable[..., Any]

    """

    def wrapper(func: Callable[..., Any]) -> Any:
        if (schedule and repeat_pattern) or (not schedule and not repeat_pattern):
            raise Exception(
                "Only exactly 1 of (repeat_pattern, schedule) can be passed in"
            )
        name = _symbols.get_symbol_table().register(
            func, schedule=schedule, repeat_pattern=repeat_pattern
        )
        _instrumentation.get_tracer().cronjob_loaded(
            name, schedule=schedule, repeat_pattern=repeat_pattern
        )
        return func

    return wrapper


def get_jobs() -> List[remote_pb2.CronJob]:
    cron_jobs = []
    for cronjob in _symbols.get_symbol_table().get_cronjobs():
        schedule_job = cronjob.repeat_pattern

        target_time = ""
        target_day_of_week = remote_pb2.DayOfWeek.UNKNOWN_DAY
        unit = remote_pb2.Unit.UNKNOWN_UNIT
        interval = 0
        schedule_rule = ""

        if schedule_job:
            unit = remote_pb2.Unit.Value(schedule_job.unit.upper())
            interval = schedule_job.interval
            if schedule_job.at_time is not None:
                target_time = schedule_job.at_time.isoformat()
            if schedule_job.start_day is not None:
                target_day_of_week = remote_pb2.DayOfWeek.Value(
                    schedule_job.start_day.upper()
                )
        elif cronjob.schedule:
            schedule_rule = cronjob.schedule
        else:
            raise Exception("Expected repeat_pattern or schedule to be set")

        cron_jobs.append(
            remote_pb2.CronJob(
                qualified_symbol=cronjob.name,
                target_time=target_time,
                target_day_of_week=target_day_of_week,
                unit=unit,
                interval=interval,
                schedule_rule=schedule_rule,
            )
        )

    return cron_jobs


def pretty_print(self: remote_pb2.CronJob) -> str:
    s = f"Function: {self.qualified_symbol}"
    s += f"\nInterval: every {self.interval} {remote_pb2.Unit.Name(self.unit).lower()}"
    if self.target_time:
        s += f"\n at {self.target_time}"
    if self.target_day_of_week:
        s += f"\n on {remote_pb2.DayOfWeek.Name(self.target_day_of_week).capitalize()}"
    return s



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: jetpack/proto/runtime/v1alpha1/remote.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+jetpack/proto/runtime/v1alpha1/remote.proto\x12\x06remote\x1a\x1fgoogle/protobuf/timestamp.proto\"[\n\x11RemoteCallRequest\x12)\n\x10qualified_symbol\x18\x01 \x01(\tR\x0fqualifiedSymbol\x12\x1b\n\tjson_args\x18\x02 \x01(\tR\x08jsonArgs\"7\n\x12RemoteCallResponse\x12!\n\x0cjson_results\x18\x01 \x01(\tR\x0bjsonResults\"Z\n\x11\x43reateTaskRequest\x12 \n\x04task\x18\x01 \x01(\x0b\x32\x0c.remote.TaskR\x04task\x12#\n\x0bsdk_version\x18\x02 \x01(\tB\x02\x18\x01R\nsdkVersion\"-\n\x12\x43reateTaskResponse\x12\x17\n\x07task_id\x18\x01 \x01(\tR\x06taskId\",\n\x11\x43\x61ncelTaskRequest\x12\x17\n\x07task_id\x18\x01 \x01(\tR\x06taskId\".\n\x12\x43\x61ncelTaskResponse\x12\x18\n\x07success\x18\x01 \x01(\x08R\x07success\"T\n\x11PostResultRequest\x12\x17\n\x07\x65xec_id\x18\x01 \x01(\tR\x06\x65xecId\x12&\n\x06result\x18\x02 \x01(\x0b\x32\x0e.remote.ResultR\x06result\"\x14\n\x12PostResultResponse\"\xff\x02\n\x04Task\x12\x1c\n\tnamespace\x18\x03 \x01(\tR\tnamespace\x12\x1e\n\x08hostname\x18\x06 \x01(\tB\x02\x18\x01R\x08hostname\x12\x19\n\x08\x61pp_name\x18\n \x01(\tR\x07\x61ppName\x12%\n\x0e\x63hart_revision\x18\x0b \x01(\tR\rchartRevision\x12)\n\x10qualified_symbol\x18\x02 \x01(\tR\x0fqualifiedSymbol\x12!\n\x0c\x65ncoded_args\x18\x04 \x01(\x0cR\x0b\x65ncodedArgs\x12$\n\x0eparent_task_id\x18\x05 \x01(\tR\x0cparentTaskId\x12;\n\x0btarget_time\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ntargetTime\x12-\n\x12with_checkpointing\x18\t \x01(\x08R\x11withCheckpointing\x12\x17\n\x07task_id\x18\x08 \x01(\tR\x06taskId\"R\n\x06Result\x12#\n\x05value\x18\x01 \x01(\x0b\x32\r.remote.ValueR\x05value\x12#\n\x05\x65rror\x18\x02 \x01(\x0b\x32\r.remote.ErrorR\x05\x65rror\"m\n\x05\x45rror\x12%\n\x04\x63ode\x18\x01 \x01(\x0e\x32\x11.remote.ErrorCodeR\x04\x63ode\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\x12#\n\rencoded_error\x18\x03 \x01(\x0cR\x0c\x65ncodedError\",\n\x05Value\x12#\n\rencoded_value\x18\x01 \x01(\x0cR\x0c\x65ncodedValue\"\xb0\x02\n\x12RegisterAppRequest\x12\x1c\n\tnamespace\x18\x01 \x01(\tR\tnamespace\x12\x1a\n\x08hostname\x18\x02 \x01(\tR\x08hostname\x12,\n\tcron_jobs\x18\x04 \x03(\x0b\x32\x0f.remote.CronJobR\x08\x63ronJobs\x12+\n\x11qualified_symbols\x18\x05 \x03(\tR\x10qualifiedSymbols\x12G\n\tendpoints\x18\x06 \x03(\x0b\x32).remote.RegisterAppRequest.EndpointsEntryR\tendpoints\x1a<\n\x0e\x45ndpointsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\x15\n\x13RegisterAppResponse\"P\n\x14\x44\x65registerAppRequest\x12\x1c\n\tnamespace\x18\x01 \x01(\tR\tnamespace\x12\x1a\n\x08hostname\x18\x02 \x01(\tR\x08hostname\"\x17\n\x15\x44\x65registerAppResponse\"/\n\x14WaitForResultRequest\x12\x17\n\x07task_id\x18\x01 \x01(\tR\x06taskId\"?\n\x15WaitForResultResponse\x12&\n\x06result\x18\x01 \x01(\x0b\x32\x0e.remote.ResultR\x06result\":\n\x1cGetSymbolFromEndpointRequest\x12\x1a\n\x08\x65ndpoint\x18\x01 \x01(\tR\x08\x65ndpoint\"7\n\x1dGetSymbolFromEndpointResponse\x12\x16\n\x06symbol\x18\x01 \x01(\tR\x06symbol\"\xf8\x01\n\x07\x43ronJob\x12)\n\x10qualified_symbol\x18\x03 \x01(\tR\x0fqualifiedSymbol\x12\x1f\n\x0btarget_time\x18\x04 \x01(\tR\ntargetTime\x12>\n\x12target_day_of_week\x18\x05 \x01(\x0e\x32\x11.remote.DayOfWeekR\x0ftargetDayOfWeek\x12 \n\x04unit\x18\x06 \x01(\x0e\x32\x0c.remote.UnitR\x04unit\x12\x1a\n\x08interval\x18\x07 \x01(\x03R\x08interval\x12#\n\rschedule_rule\x18\x08 \x01(\tR\x0cscheduleRule\")\n\x0eGetTaskRequest\x12\x17\n\x07task_id\x18\x01 \x01(\tR\x06taskId\"D\n\x0fGetTaskResponse\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12!\n\x0c\x65ncoded_args\x18\x02 \x01(\x0cR\x0b\x65ncodedArgs*6\n\tErrorCode\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06SYSTEM\x10\x64\x12\x10\n\x0b\x41PPLICATION\x10\xc8\x01*R\n\x04Unit\x12\x10\n\x0cUNKNOWN_UNIT\x10\x00\x12\x0b\n\x07SECONDS\x10\x01\x12\x0b\n\x07MINUTES\x10\x02\x12\t\n\x05HOURS\x10\x03\x12\x08\n\x04\x44\x41YS\x10\x04\x12\t\n\x05WEEKS\x10\x05*x\n\tDayOfWeek\x12\x0f\n\x0bUNKNOWN_DAY\x10\x00\x12\n\n\x06SUNDAY\x10\x01\x12\n\n\x06MONDAY\x10\x02\x12\x0b\n\x07TUESDAY\x10\x03\x12\r\n\tWEDNESDAY\x10\x04\x12\x0c\n\x08THURSDAY\x10\x05\x12\n\n\x06\x46RIDAY\x10\x06\x12\x0c\n\x08SATURDAY\x10\x07\x32\xbc\x05\n\x0eRemoteExecutor\x12\x45\n\nCreateTask\x12\x19.remote.CreateTaskRequest\x1a\x1a.remote.CreateTaskResponse\"\x00\x12\x45\n\nCancelTask\x12\x19.remote.CancelTaskRequest\x1a\x1a.remote.CancelTaskResponse\"\x00\x12\x45\n\nPostResult\x12\x19.remote.PostResultRequest\x1a\x1a.remote.PostResultResponse\"\x00\x12N\n\rWaitForResult\x12\x1c.remote.WaitForResultRequest\x1a\x1d.remote.WaitForResultResponse\"\x00\x12<\n\x07GetTask\x12\x16.remote.GetTaskRequest\x1a\x17.remote.GetTaskResponse\"\x00\x12\x66\n\x15GetSymbolFromEndpoint\x12$.remote.GetSymbolFromEndpointRequest\x1a%.remote.GetSymbolFromEndpointResponse\"\x00\x12H\n\x0bRegisterApp\x12\x1a.remote.RegisterAppRequest\x1a\x1b.remote.RegisterAppResponse\"\x00\x12N\n\rDeregisterApp\x12\x1c.remote.DeregisterAppRequest\x1a\x1d.remote.DeregisterAppResponse\"\x00\x12\x45\n\nRemoteCall\x12\x19.remote.RemoteCallRequest\x1a\x1a.remote.RemoteCallResponse\"\x00\x42\x85\x01\n\ncom.remoteB\x0bRemoteProtoP\x01Z2go.jetpack.io/proto/jetpack/proto/runtime/v1alpha1\xa2\x02\x03RXX\xaa\x02\x06Remote\xca\x02\x06Remote\xe2\x02\x12Remote\\GPBMetadata\xea\x02\x06Remoteb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'jetpack.proto.runtime.v1alpha1.remote_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\ncom.remoteB\013RemoteProtoP\001Z2go.jetpack.io/proto/jetpack/proto/runtime/v1alpha1\242\002\003RXX\252\002\006Remote\312\002\006Remote\342\002\022Remote\\GPBMetadata\352\002\006Remote'
  _CREATETASKREQUEST.fields_by_name['sdk_version']._options = None
  _CREATETASKREQUEST.fields_by_name['sdk_version']._serialized_options = b'\030\001'
  _TASK.fields_by_name['hostname']._options = None
  _TASK.fields_by_name['hostname']._serialized_options = b'\030\001'
  _REGISTERAPPREQUEST_ENDPOINTSENTRY._options = None
  _REGISTERAPPREQUEST_ENDPOINTSENTRY._serialized_options = b'8\001'
  _ERRORCODE._serialized_start=2238
  _ERRORCODE._serialized_end=2292
  _UNIT._serialized_start=2294
  _UNIT._serialized_end=2376
  _DAYOFWEEK._serialized_start=2378
  _DAYOFWEEK._serialized_end=2498
  _REMOTECALLREQUEST._serialized_start=88
  _REMOTECALLREQUEST._serialized_end=179
  _REMOTECALLRESPONSE._serialized_start=181
  _REMOTECALLRESPONSE._serialized_end=236
  _CREATETASKREQUEST._serialized_start=238
  _CREATETASKREQUEST._serialized_end=328
  _CREATETASKRESPONSE._serialized_start=330
  _CREATETASKRESPONSE._serialized_end=375
  _CANCELTASKREQUEST._serialized_start=377
  _CANCELTASKREQUEST._serialized_end=421
  _CANCELTASKRESPONSE._serialized_start=423
  _CANCELTASKRESPONSE._serialized_end=469
  _POSTRESULTREQUEST._serialized_start=471
  _POSTRESULTREQUEST._serialized_end=555
  _POSTRESULTRESPONSE._serialized_start=557
  _POSTRESULTRESPONSE._serialized_end=577
  _TASK._serialized_start=580
  _TASK._serialized_end=963
  _RESULT._serialized_start=965
  _RESULT._serialized_end=1047
  _ERROR._serialized_start=1049
  _ERROR._serialized_end=1158
  _VALUE._serialized_start=1160
  _VALUE._serialized_end=1204
  _REGISTERAPPREQUEST._serialized_start=1207
  _REGISTERAPPREQUEST._serialized_end=1511
  _REGISTERAPPREQUEST_ENDPOINTSENTRY._serialized_start=1451
  _REGISTERAPPREQUEST_ENDPOINTSENTRY._serialized_end=1511
  _REGISTERAPPRESPONSE._serialized_start=1513
  _REGISTERAPPRESPONSE._serialized_end=1534
  _DEREGISTERAPPREQUEST._serialized_start=1536
  _DEREGISTERAPPREQUEST._serialized_end=1616
  _DEREGISTERAPPRESPONSE._serialized_start=1618
  _DEREGISTERAPPRESPONSE._serialized_end=1641
  _WAITFORRESULTREQUEST._serialized_start=1643
  _WAITFORRESULTREQUEST._serialized_end=1690
  _WAITFORRESULTRESPONSE._serialized_start=1692
  _WAITFORRESULTRESPONSE._serialized_end=1755
  _GETSYMBOLFROMENDPOINTREQUEST._serialized_start=1757
  _GETSYMBOLFROMENDPOINTREQUEST._serialized_end=1815
  _GETSYMBOLFROMENDPOINTRESPONSE._serialized_start=1817
  _GETSYMBOLFROMENDPOINTRESPONSE._serialized_end=1872
  _CRONJOB._serialized_start=1875
  _CRONJOB._serialized_end=2123
  _GETTASKREQUEST._serialized_start=2125
  _GETTASKREQUEST._serialized_end=2166
  _GETTASKRESPONSE._serialized_start=2168
  _GETTASKRESPONSE._serialized_end=2236
  _REMOTEEXECUTOR._serialized_start=2501
  _REMOTEEXECUTOR._serialized_end=3201
# @@protoc_insertion_point(module_scope)



"""
Submodule for running, scheduling, and managing Jetroutines
"""

from typing import Dict

from jetpack.core._task import Task

__pdoc__: Dict[str, bool] = {}

__pdoc__["_jetroutine"] = True
__pdoc__["_task"] = True
